// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include <hu/varadiistvan/scpl/io/Waiter.h>
#include <hu/varadiistvan/scpl/io/LocalServerSocket.h>
#include <hu/varadiistvan/scpl/io/LocalAcceptor.h>
#include <hu/varadiistvan/scpl/io/BufferedStream.h>
#include <hu/varadiistvan/scpl/io/DataStream.h>

#include <hu/varadiistvan/scpl/Thread.h>

#include <cstdio>

//------------------------------------------------------------------------------

using hu::varadiistvan::scpl::io::Waiter;
using hu::varadiistvan::scpl::io::LocalServerSocket;
using hu::varadiistvan::scpl::io::LocalAcceptor;
using hu::varadiistvan::scpl::io::BufferedStream;
using hu::varadiistvan::scpl::io::DataStream;

using hu::varadiistvan::scpl::Thread;

//------------------------------------------------------------------------------

class ServerThread : public Thread
{
    hu::varadiistvan::scpl::io::Waiter waiter;

    hu::varadiistvan::scpl::io::BufferedStream* bufferedStream;

    hu::varadiistvan::scpl::io::DataStream stream;

public:
    ServerThread(hu::varadiistvan::scpl::io::LocalAcceptor& acceptor);

    virtual void run();
};

//------------------------------------------------------------------------------

ServerThread::ServerThread(LocalAcceptor& acceptor) :
    bufferedStream(acceptor.getSocket(&waiter)),
    stream(*bufferedStream)
{
    printf("Created ServerThread[%p]\n", this);
}

//------------------------------------------------------------------------------

void ServerThread::run()
{
    while(stream) {
        uint8_t value = stream.readU8();
        if (!stream) continue;

        printf("ServerThread[%p]: value=0x%02x\n", this, value);
    }
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

class ListenThread : public Thread
{
public:
    virtual void run();
};


//------------------------------------------------------------------------------

void ListenThread::run()
{
    printf("ListenThread::run\n");

    Waiter waiter;

    LocalServerSocket socket("testmc", &waiter);

    LocalAcceptor& acceptor = socket.getAcceptor();
    while(!acceptor.failed()) {
        printf("ListenThread::run: accepting...\n");
        while (acceptor.accept()) {
            printf("ListenThread::run: connection accepted\n");
            ServerThread* serverThread = new ServerThread(acceptor);
            serverThread->start();
        }
        printf("ListenThread::run: waiting...\n");
        waiter.wait();
        printf("ListenThread::run: waited...\n");
    }
}

//------------------------------------------------------------------------------

int main()
{
    ListenThread listenThread;
    listenThread.start();

    while(true) {
        Thread::sleep(3600*1000);
    };
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
